window.Gleam.initSearch([{"doc":"cx","title":"cx","content":"# cx\n\n[![Package Version](https://img.shields.io/hexpm/v/cx)](https://hex.pm/packages/cx)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/cx/)\n\nA library, written in [Gleam](https://gleam.run/), to help create data\nstructures containing different data types and levels of nesting.\n\n**Note**: This library is still being written!\n\n## Background\n\nGleam is statically typed, and has certain restrictions for certain data types.\nFor example, a [Dict](https://hexdocs.pm/gleam_stdlib/gleam/dict.html) in Gleam\nhas this restriction on the keys and values:\n\n```\n...all the keys must be of the same type and all the values must be of the same type.\n```\n\n[Lists](https://hexdocs.pm/gleam_stdlib/gleam/list.html) have a similar restriction:\n\n```\nAll elements of a list must be the same type\n```\n\nWhat if we want to represent more complicated, nested data structures as well?\nFor example, here is a native data structure in Python:\n\n```python3\ndata = {\n    &quot;settings&quot;: {\n        &quot;theme&quot;: &quot;dark&quot;,\n        &quot;themes: [&quot;light&quot;, &quot;dark&quot;]\n    }\n    &quot;people&quot;: [\n        {&quot;Nicknames&quot;: [&quot;Jane&quot;, &quot;Jill&quot;]},\n    ]\n}\n```\n\nCan we do this in Gleam? Yes, we can, with\n[records](https://tour.gleam.run/data-types/records/).\nRecords can also help us know the type of each object stored in the data\nstructure, which is significant because runtime reflection is not available in\nGleam.\n\nThis library was written to be used with an HTML templating engine, where pieces\nof the data structure are used to replace variables (e.g. `@person.name`)in the\nHTML.\n\n## Installing\n```sh\ngleam add cx\n```\n\n## Usage\n\n```gleam\nimport cx\n\npub fn main() {\n  let context =\n    cx.dict()\n    |&gt; cx.add(\n      &quot;settings&quot;,\n      cx.dict()\n        |&gt; cx.add_string(&quot;theme&quot;, &quot;dark&quot;)\n        |&gt; cx.add_strings(&quot;themes&quot;, [&quot;light&quot;, &quot;dark&quot;]),\n    )\n    |&gt; cx.add_list(&quot;people&quot;, [\n      cx.add_strings(cx.dict(), &quot;Nicknames&quot;, [&quot;Jane&quot;, &quot;Jill&quot;]),\n    ])\n\n\n    // Result: Ok([&quot;light&quot;, &quot;dark&quot;])\n    cx.get_strings(context, &quot;settings.themes&quot;)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/cx&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"cx","title":"Context","content":"pub type Context {\n  CString(value: String)\n  CInt(value: Int)\n  CFloat(value: Float)\n  CDict(value: Dict(String, Context))\n  CList(value: List(Context))\n}\n A recursive type for creating and reading nested objects (lists and\n dictionaries) that contain different combinations of types.\nCString(value: String)\n\n\nCInt(value: Int)\n\n\nCFloat(value: Float)\n\n\nCDict(value: Dict(String, Context))\n\n\nCList(value: List(Context))\n\n\nSynonyms:\ncx.Context\ncx Context","url":"cx.html#Context"},{"doc":"cx","title":"ContextError","content":"pub type ContextError {\n  KeyNotFound(key: String)\n  ValueNotFound(key: String)\n  UnexpectedContextType(key: String)\n}\n Errors that can occur when reading values from the &quot;context&quot; (a data\n structure).\nKeyNotFound(key: String)\n\n\nValueNotFound(key: String)\n\n\nUnexpectedContextType(key: String)\n\n\nSynonyms:\ncx.ContextError\ncx ContextError","url":"cx.html#ContextError"},{"doc":"cx","title":"add","content":"pub fn add(\n  context: Context,\n  key: String,\n  value: Context,\n) -&gt; Context\n Add a value, of any type, under a string key, to the &quot;context&quot; (a data\n structure).\nSynonyms:\ncx.add\ncx add","url":"cx.html#add"},{"doc":"cx","title":"add_float","content":"pub fn add_float(\n  context: Context,\n  key: String,\n  value: Float,\n) -&gt; Context\n Add a float value, stored under a certain key, to the &quot;context&quot; (a data\n structure).\nSynonyms:\ncx.add_float\ncx add_float","url":"cx.html#add_float"},{"doc":"cx","title":"add_int","content":"pub fn add_int(\n  context: Context,\n  key: String,\n  value: Int,\n) -&gt; Context\n Add an integer value, stored under a certain key, to the &quot;context&quot; (a data\n structure).\nSynonyms:\ncx.add_int\ncx add_int","url":"cx.html#add_int"},{"doc":"cx","title":"add_list","content":"pub fn add_list(\n  context: Context,\n  key: String,\n  value: List(Context),\n) -&gt; Context\n Add a list of values, of any type, under a string key, to the &quot;context&quot; (a\n data structure).\nSynonyms:\ncx.add_list\ncx add_list","url":"cx.html#add_list"},{"doc":"cx","title":"add_string","content":"pub fn add_string(\n  context: Context,\n  key: String,\n  value: String,\n) -&gt; Context\n Add a string value, stored under a certain key, to the &quot;context&quot; (a data\n structure).\nSynonyms:\ncx.add_string\ncx add_string","url":"cx.html#add_string"},{"doc":"cx","title":"add_strings","content":"pub fn add_strings(\n  context: Context,\n  key: String,\n  values: List(String),\n) -&gt; Context\n Store a list of strings, under a string key, in the &quot;context&quot; (a data\n structure).\nSynonyms:\ncx.add_strings\ncx add_strings","url":"cx.html#add_strings"},{"doc":"cx","title":"dict","content":"pub fn dict() -&gt; Context\n Create a data structure to store key-value pairs, where the key is a String.\nSynonyms:\ncx.dict\ncx dict","url":"cx.html#dict"},{"doc":"cx","title":"get","content":"pub fn get(\n  context: Context,\n  key: String,\n) -&gt; Result(Context, ContextError)\n Get a value, of any type, stored under a certain key, from the &quot;context&quot; (a\n data structure).\nSynonyms:\ncx.get\ncx get","url":"cx.html#get"},{"doc":"cx","title":"get_list","content":"pub fn get_list(\n  context: Context,\n  key: String,\n) -&gt; Result(List(Context), ContextError)\n Returns a list of values, of varying types, stored under a string key in\n the &quot;context&quot; (a data structure).\nSynonyms:\ncx.get_list\ncx get_list","url":"cx.html#get_list"},{"doc":"cx","title":"get_string","content":"pub fn get_string(\n  context: Context,\n  key: String,\n) -&gt; Result(String, ContextError)\n Get a string value, stored under a string key, from the &quot;context&quot; (a\n data structure).\nSynonyms:\ncx.get_string\ncx get_string","url":"cx.html#get_string"},{"doc":"cx","title":"get_strings","content":"pub fn get_strings(\n  context: Context,\n  key: String,\n) -&gt; Result(List(String), ContextError)\n Get a list of strings stored in the context under a given string key.\nSynonyms:\ncx.get_strings\ncx get_strings","url":"cx.html#get_strings"},{"doc":"cx","title":"cx","content":" The &quot;Context&quot; data type exists to help create and read different types of\n data structures.\n\n For example, to store a key and value:\n\n ```gleam\n let context = cx.dict()\n    |&gt; cx.add_string(&quot;first_name&quot;, first_name)\n ```\n\n To retrieve that value:\n\n ```gleam\n cx.get_string(context, &quot;first_name&quot;)\n ```\n\n To store a nested key and value:\n\n ```gleam\n  let context =\n    cx.dict()\n    |&gt; cx.add(\n     &quot;person&quot;,\n     cx.dict()\n       |&gt; cx.add_string(&quot;first_name&quot;, first_name),\n   )\n ```\n\n To retrieve that value:\n\n ```gleam\n cx.get_string(context, &quot;person.first_name&quot;)\n ```\n","url":"cx.html"}]);